<?php
if ( !class_exists( 'pmPostController' ) ) :
    class pmPostController {

        function __construct() {

            add_action( 'add_meta_boxes', [ $this, 'add_custom_meta_box' ] );
            add_action( 'admin_enqueue_scripts', [ $this, 'meta_option_script' ] );
            add_action( 'wp_ajax_pm_img_preview', [ $this, 'pm_img_preview' ] );
            add_action( 'wp_ajax_pm_audio_preview', [ $this, 'pm_audio_preview' ] );
            add_action( 'wp_ajax_pm_video_preview', [ $this, 'pm_video_preview' ] );
            add_action( 'wp_ajax_pm_file_preview', [ $this, 'pm_file_preview' ] );
            add_action( 'wp_ajax_pm_load_group', [ $this, 'pm_load_group' ] );
            add_action( 'save_post', [ $this, 'save_meta_option_data' ] );
            add_action( 'admin_head-media-upload-popup', [ $this, 'popup_head' ] );
        }

        function popup_head() {
            // phpcs:ignore WordPress.Security.NonceVerification.Recommended
            if ( isset( $_GET['pm_media_file'] ) ) {
                ?>
                <script>
                    (function ($) {
                        $(function () {
                            $('form#filter').each(function () {
                                $(this).append('<input type="hidden" name="pm_media_file" value="1" />');
                            });

                            $('#media-items').bind('DOMNodeInserted', function () {
                                var $this = $(this);
                                $this.find('tr.image_alt').hide();
                                $this.find('tr.post_excerpt').hide();
                                $this.find('tr.url').hide();
                                $this.find('tr.align').hide();
                                $this.find('tr.image-size').hide();
                                $this.find('tr.submit input.button').val('<?php esc_html__( 'Use This File', 'post-meta' ); ?>');
                            }).trigger('DOMNodeInserted');
                        });
                    })(jQuery);
                </script>
                <?php
            }
        }

        function meta_option_script() {
            global $post_type, $postMeta;;

            // phpcs:ignore WordPress.Security.NonceVerification.Recommended
            if ( !empty( $_GET['post_type'] ) ) :

                if ( !post_type_supports( $post_type, 'editor' ) ) {
                    add_thickbox();
                    wp_enqueue_script( 'media-upload' );
                    add_action( 'admin_print_footer_scripts', 'wp_tiny_mce', 25 );
                }

                wp_enqueue_script( 'jquery' );
                wp_enqueue_script( 'jquery-ui-core' );
                wp_enqueue_script( 'jquery-ui-widget' );
                wp_enqueue_script( 'jquery-ui-mouse' );
                wp_enqueue_script( 'jquery-ui-slider' );

                wp_enqueue_style( 'pluginCore-css', PM_ASSECTS_URL . 'css/pluginCore.css', '', $postMeta->version );
                wp_enqueue_style( 'post-meta-post-style', PM_ASSECTS_URL . 'css/post_meta_post.css', '', $postMeta->version );

                $data = [
                    'on_preview'      => PM_ASSECTS_URL . 'images/no_preview.png',
                    'uploaderURL'     => PM_PLUGIN_URL . '/lib/helpers/uploader/uploader.php',
                    'uploadurl'       => site_url() . '/wp-content/uploads',
                    'short_code_icon' => PM_ASSECTS_URL . 'images/icon.png'
                ];
                wp_localize_script( 'jquery', 'pm_jsvar', $data );


                wp_enqueue_script( 'jquery-ui-datepicker', PM_ASSECTS_URL . 'js/ui/jquery.ui.datepicker.js', [ 'jquery' ], $postMeta->version, true );
                wp_enqueue_script( 'jquery-ui-timepicker-addon', PM_ASSECTS_URL . 'js/ui/jquery-ui-timepicker-addon.js', [ 'jquery', 'jquery-ui-slider', 'jquery-ui-datepicker' ], $postMeta->version, true );
                wp_enqueue_style( 'jquery-ui-css', PM_ASSECTS_URL . 'css/jquery/jquery-ui.min.css', '', $postMeta->version );
                wp_enqueue_style( 'post-meta-validationEngine-css', PM_ASSECTS_URL . 'css/jquery/validationEngine.css', '', $postMeta->version );

                wp_enqueue_script( 'post-meta-validationEngine', PM_ASSECTS_URL . 'js/jquery/validationEngine.js', [ 'jquery' ], $postMeta->version, true );
                wp_enqueue_script( 'post-meta-validationEngine-en', PM_ASSECTS_URL . 'js/jquery/validationEngine-en.js', [ 'post-meta-validationEngine' ], $postMeta->version, true );
                wp_enqueue_script( 'post-meta-admin-script', PM_ASSECTS_URL . 'js/post_meta_admin.js', [ 'post-meta-validationEngine' ], $postMeta->version, true );

                $screen = get_current_screen();
                if ( $screen->base != "edit" ) {
                    wp_enqueue_script( 'post-meta-post-script', PM_ASSECTS_URL . 'js/post_meta_post.js', [ 'jquery', 'media-upload', 'thickbox', 'jquery-fileuploader' ], $postMeta->version, true );
                }

            endif;
        }

        function pm_img_preview() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) ) die( 'Security check' );
            $postMeta->render( "thumb", [
                'file'   => !empty( $_REQUEST['imgurl'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['imgurl'] ) ) : null,
                'width'  => 150,
                'height' => 150
            ] );
            die();

        }

        function pm_audio_preview() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) ) die( 'Security check' );
            $postMeta->render( "preview_audio", [
                    'file' => !empty( $_REQUEST['url'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['url'] ) ) : null,
                    'id'   => !empty( $_REQUEST['id'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['id'] ) ) : null,
                ]
            );
            die();

        }

        function pm_video_preview() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) ) die( 'Security check' );
            $id = !empty( $_REQUEST['id'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['id'] ) ) : null;
            $url = !empty( $_REQUEST['url'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['url'] ) ) : null;
            $postMeta->render( "preview_video", [
                    'file' => $url,
                    'id'   => $id
                ]
            );
            die();
        }

        function pm_file_preview() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) ) die( 'Security check' );
            // phpcs:ignore PluginCheck.CodeAnalysis.ImageFunctions.NonEnqueuedImage
            ?>
            <img alt="Folder" src='<?php echo esc_url( PM_ASSECTS_URL ); ?>images/folder.png'/>
            <?php
            die();
        }

        function pm_load_group() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) ) die( 'Security check' );
            $_postType = !empty( $_REQUEST['post_type'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['post_type'] ) ) : null;
            $pm_options = get_option( $postMeta->options['post_meta'] );
            $postType = sanitize_title( $pm_options['post_type'] );
            $groupKey = sanitize_title( $pm_options['group_key'] );
            $group = $pm_options[$_postType]['group'][$groupKey];
            $groupCount = !empty( $pm_options['group_count'] ) ? absint( $pm_options['group_count'] ) : 0;
            ?>
            <div class='pm_group'>
                <?php foreach ( $group['field'] as $field ) {
                    if ( $field['duplicate'] ) { ?>
                        <div class='pm_field_group_<?php echo esc_attr( $field['meta_key'] ) ?> pm_sortable pm_field_group' style='border:none; margin:10px 0'>
                    <?php } ?>
                    <?php $postMeta->render( "postField", [ 'field'    => $field, 'group_count' => $groupCount,
                                                            'group_id' => $groupKey, 'key' => 1, 'pm_load_group' => true ] );
                    if ( $field['duplicate'] ) { ?>
                        </div>
                    <?php }
                } ?>

                <div class='pm-group-control'>
                    <a class='duplicate-add-group' data-group_id='<?php echo esc_attr( $groupKey ) ?>'
                       data-post_type='<?php echo esc_attr( $postType ) ?>' href='#Add Group'>Add New Group</a>
                    <a class='duplicate-remove-group' style='display:inline'
                       data-group_id='<?php echo esc_attr( $groupKey ) ?>'
                       href='#Remove Group'>Remove Group</a>
                    <span class='group_count_label'></span>
                </div>
                ";
            </div>
            <?php
            die();
        }

        function add_custom_meta_box() {
            global $postMeta, $post_type;


            $pm_options = get_option( $postMeta->options['post_meta'] );
            $meta_boxs = $pm_options[$post_type];
            if ( is_array( $meta_boxs ) && !empty( $meta_boxs ) ) {
                foreach ( $meta_boxs['group'] as $Key => $meta_box ) {
                    $groupKey = $Key;
                    if ( $meta_box['field'] ) {
                        add_meta_box(
                            $meta_box['meta_key'], // $id
                            $meta_box['title'], // $title
                            [ $this, 'render_meta_box_content' ], // $callback
                            $post_type, // $page
                            $meta_box['context'], // $context
                            $meta_box['priority'],
                            [ 'fields' => $meta_box['field'], 'group_id' => $groupKey, 'group_metakey' => $meta_box['meta_key'] ]
                        ); // $priority
                    }
                } // end of  $meta_boxs['group'] foreach
            }


        }

        function render_meta_box_content( $post, $arguments ) {
            wp_nonce_field( plugin_basename( __FILE__ ), 'pm_nonce' );
            global $postMeta, $post_type;

            $fields = $arguments['args']['fields'];
            $group_id = $arguments['args']['group_id'];
            $group_metakey = $arguments['args']['group_metakey'];
            if ( $fields ) {
                $group_count_meta = get_post_meta( $post->ID, 'group_count_' . $group_metakey, true );
                $group_count = ( $group_count_meta ) ? $group_count_meta : 1;
                $groupRemoveVisibility = ( $group_count > 1 ) ? 'inline' : 'none';

                for ( $i = 1; $i <= $group_count; $i++ ) {
                    echo "<div class='pm_group'>";
                    foreach ( $fields as $field ) {

                        $meta_data = get_post_meta( $post->ID, $field['meta_key'], true );
                        $metas = $meta_data[$i];

                        $border = ( is_array( $metas ) && count( $metas ) > 1 ) ? '1px dashed #000000' : 'none';

                        echo !empty( $field['duplicate'] ) ? wp_kses_post( "<div class='pm_field_group_" . esc_attr( $field['meta_key'] ) . " pm_sortable pm_field_group' style='border:" . esc_attr( $border ) . "; margin:10px 0'>" ) : '';
                        if ( is_array( $metas ) ) {
                            $last_key = count( $metas );
                            if ( $last_key > 1 ) {
                                $remove_field = true;
                                $label_count = true;
                            } else {
                                $remove_field = false;
                                $label_count = false;
                            }
                            foreach ( $metas as $key => $meta ) {
                                $postMeta->render( "postField", [ 'field' => $field, 'group_count' => $i, 'group_id' => $group_id, 'value' => $meta, 'key' => "$key", 'label_count' => $label_count, 'remove_field' => $remove_field ] );
                            }
                        } else {
                            $key = ( $group_count ) ? 1 : null;
                            $postMeta->render( "postField", [ 'field' => $field, 'group_count' => $i, 'group_id' => $group_id, 'value' => $metas, 'key' => "$key" ] );
                        }
                        echo !empty( $field['duplicate'] ) ? "</div>" : '';
                    }

                    $html = "<div class='pm-group-control'>
                                        <a class='duplicate-add-group' group_id='" . esc_attr( $group_id ) . "' post_type='" . esc_attr( $post_type ) . "' meta_key='" . esc_attr( $group_metakey ) . "' href='#'>Add New Group</a>
                                        <a class='duplicate-remove-group' style='display:" . esc_attr( $groupRemoveVisibility ) . "' group_id='" . esc_attr( $group_id ) . "' href='#'>Remove Group</a>
                                        <span class='group_count_label'>$i</span>
                                      </div>";

                    if ( $group_id )
                        PmFns::print_html( $html );
                    echo "</div>";
                }
                echo "<input type='hidden' name='mofields[group_count_" . esc_attr( $group_metakey ) . "]' id='group_count' value='" . esc_attr( $group_count ) . "' />";
            }

        }

        function save_meta_option_data( $post_id ) {

            // verify if this is an auto save routine.
            // If it is our form has not been submitted, so we dont want to do anything

            if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
                return;

            // verify this came from the our screen and with proper authorization,
            // because save_post can be triggered at other times
            $nonce = !empty( $_POST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_POST['pm_nonce'] ) ) : '';
            if ( !wp_verify_nonce( $nonce, plugin_basename( __FILE__ ) ) ) return;

            // Check permissions
            if ( !empty( $_GET['post_type'] ) ) {
                if ( !current_user_can( 'edit_' . sanitize_text_field( wp_unslash( $_GET['post_type'] ) ), $post_id ) )
                    return;
            }
            // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.MissingUnslash, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
            $customFields = !empty( $_POST['mofields'] ) && is_array( $_POST['mofields'] ) ? $_POST['mofields'] : [];

            foreach ( $customFields as $meta_key => $value ) {
                if ( is_array( $value ) ) {
                    $n = 1;
                    $data = [];
                    foreach ( $value as $key => $val ) {
                        $data[$n] = is_array( $val ) ? array_map( function ( $_item ) {
                            return is_array( $_item ) ? array_map( function ( $__item ) {
                                return sanitize_text_field( wp_unslash( $__item ) );
                            }, $_item ) : $_item;
                        }, $val ) : sanitize_text_field( wp_unslash( $val ) );
                        $n = $n + 1;
                    }
                    update_post_meta( $post_id, $meta_key, $data );
                } else {
                    $new = sanitize_text_field( wp_unslash( $value ) );
                    update_post_meta( $post_id, $meta_key, $new );
                }
            }
        }
    }
endif;
