<?php

if ( !class_exists( 'pmPostMetaController' ) ) :
    class pmPostMetaController {

        function __construct() {
            add_action( 'admin_menu', [ $this, 'post_meta_menu' ] );
            add_action( 'wp_ajax_pm_manage_group', [ $this, 'post_meta_manage_group' ] );
            add_action( 'wp_ajax_pm_add_group', [ $this, 'post_meta_add_group' ] );
            add_action( 'wp_ajax_pm_add_field', [ $this, 'post_meta_add_field' ] );
            add_action( 'wp_ajax_pm_change_field', [ $this, 'post_meta_change_field' ] );
            add_action( 'wp_ajax_meta_option_group_save', [ $this, 'post_meta_group_save' ] );
        }


        function post_meta_menu() {
            $page = add_menu_page( 'Post Meta', 'Post Meta', 'administrator', 'post-meta', [ $this, 'post_meta_menu_page' ], PM_ASSECTS_URL . 'images/icon.png', 52 );
            $page = add_submenu_page( 'post-meta', 'Manage Post Meta', 'Post Meta', 'administrator', 'post-meta', [ $this, 'post_meta_menu_page' ] );

            add_action( 'admin_print_styles-' . $page, [ $this, 'post_meta_sc' ] );
        }

        function post_meta_sc() {
            global $postMeta;
            wp_enqueue_script( 'jquery-ui-tabs' );
            wp_enqueue_script( 'jquery-ui-sortable' );
            wp_enqueue_script( 'jquery-ui-draggable' );
            wp_enqueue_script( 'jquery-ui-droppable' );
            wp_enqueue_script( 'jquery-ui-selectable' );
            wp_enqueue_script( 'jquery-ui-resizable' );
            wp_enqueue_script( 'jquery-ui-dialog' );

            wp_enqueue_style( 'jquery-ui-css', PM_ASSECTS_URL . 'css/jquery/jquery-ui.min.css', '', $postMeta->version );
            wp_enqueue_style( 'pluginCore-style', PM_ASSECTS_URL . 'css/pluginCore.css', '', $postMeta->version );
            wp_enqueue_style( 'post-meta-admin-style', PM_ASSECTS_URL . 'css/post_meta_admin.css', '', $postMeta->version );
            wp_enqueue_style( 'post-meta-validationEngine-css', PM_ASSECTS_URL . 'css/jquery/validationEngine.css', '', $postMeta->version );

            wp_enqueue_script( 'post-meta-validationEngine', PM_ASSECTS_URL . 'js/jquery/validationEngine.js', [ 'jquery' ], $postMeta->version, true );
            wp_enqueue_script( 'post-meta-validationEngine-en', PM_ASSECTS_URL . 'js/jquery/validationEngine-en.js', [ 'post-meta-validationEngine' ], $postMeta->version, true );
            wp_enqueue_script( 'post-meta-third-party-slug-string', PM_ASSECTS_URL . 'js/jquery/jquery.stringToSlug.js', '', $postMeta->version, true );
            wp_enqueue_script( 'post-meta-admin-script', PM_ASSECTS_URL . 'js/post_meta_admin.js', [ 'post-meta-validationEngine' ], $postMeta->version, true );
        }

        function post_meta_menu_page() {
            global $postMeta;
            $postMeta->getRender( 'manage', '', 'postmeta' );
        }

        function post_meta_manage_group() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) ) die( 'Security check' );
            $postMeta->getRender( 'manageForm', [
                'post_type' => !empty( $_POST['post_type'] ) ? sanitize_text_field( wp_unslash( $_POST['post_type'] ) ) : null,
            ], 'postmeta' );
            die();
        }

        function post_meta_add_group() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) ) die( 'Security check' );
            $id = !empty( $_POST['id'] ) ? sanitize_text_field( wp_unslash( $_POST['id'] ) ) : null;
            $meta_box['id'] = "group_" . $id;
            $meta_box['title'] = "New Group ";
            $postType = !empty( $_POST['post_type'] ) ? sanitize_text_field( wp_unslash( $_POST['post_type'] ) ) : null;
            $postMeta->getRender( 'group',
                [
                    'meta_box'  => $meta_box,
                    'id'        => $id,
                    'toggle'    => !empty( $_POST['toggle'] ),
                    'post_type' => $postType
                ],
                'postmeta' );

            die();
        }

        function post_meta_add_field() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) ) die( 'Security check' );
            $postType = !empty( $_POST['post_type'] ) ? sanitize_text_field( wp_unslash( $_POST['post_type'] ) ) : null;
            $field_id = !empty( $_POST['field_id'] ) ? sanitize_text_field( wp_unslash( $_POST['field_id'] ) ) : null;
            $group_id = !empty( $_POST['group_id'] ) ? sanitize_text_field( wp_unslash( $_POST['group_id'] ) ) : null;
            $field_type = !empty( $_POST['field_type'] ) ? sanitize_text_field( wp_unslash( $_POST['field_type'] ) ) : null;
            $field_type_data = $postMeta->getFields( 'key' );
            $field = [
                'title' => $field_type_data[$field_type] . " Field",
                'type'  => $field_type,
            ];
            $postMeta->getRender( "field", [
                'field'      => $field,
                'group_id'   => $group_id,
                'id'         => $field_id,
                'post_type'  => $postType,
                'field_type' => $field_type
            ], 'postmeta' );

            die();
        }

        function post_meta_change_field() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) ) die( 'Security check' );
            $data = $_REQUEST;
            $group_id_array = array_keys( $data['group'] );
            $group_id = $group_id_array[0];
            $field_id_array = array_keys( $data['group'][$group_id]['field'] );
            $field_id = $field_id_array[0];
            $field = $data['group'][$group_id]['field'][$field_id];
            $field_type = !empty( $_POST['field_type'] ) ? sanitize_text_field( wp_unslash( $_POST['field_type'] ) ) : null;
            $postMeta->getRender( "field", [
                'field'      => $field,
                'group_id'   => $group_id,
                'id'         => $field_id,
                'field_type' => $field_type,
                'toggle'     => true
            ], 'postmeta' );

            die();
        }


        function post_meta_group_save() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) || !current_user_can( 'manage_options' ) ) {
                wp_send_json_error( [ 'msg' => $postMeta->showMessage( 'Unauthorized action.', 'error' ) ] );
            }
            $post_type = !empty( $_POST['post_type'] ) ? sanitize_text_field( wp_unslash( $_POST['post_type'] ) ) : null;
            if ( !post_type_exists( $post_type ) ) {
                wp_send_json_error( [ 'msg' => $postMeta->showMessage( "Post type is not exist!!", 'error' ) ] );
            }

            // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.MissingUnslash, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
            $rawGroups = !empty( $_REQUEST['group'] ) && is_array( $_REQUEST['group'] ) ? $_REQUEST['group'] : [];
            $groups = [];
            $fieldOptions = [ 'title',
                              'type',
                              'meta_key',
                              'desc',
                              'options',
                              'id',
                              'size',
                              'max_char',
                              'css_class',
                              'css_style',
                              'allowed_extension',
                              'max_file_size',
                              'required',
                              'duplicate',
                              'register_only',
                              'alignment',
                              'multiple',
                              'width',
                              'height',
                              'min_nimber',
                              'max_number',
                              'datetime_selection',
                              'hidden_value',
                              'allowed_extension_custom',
                              'default_value'
            ];
            foreach ( $rawGroups as $rawGroup ) {
                $_group = [
                    'title'     => !empty( $rawGroup['title'] ) ? sanitize_text_field( wp_unslash( $rawGroup['title'] ) ) : null,
                    'context'   => !empty( $rawGroup['context'] ) && in_array( $rawGroup['context'], [ 'normal', 'advanced', 'side' ] ) ? $rawGroup['context'] : 'normal',
                    'priority'  => !empty( $rawGroup['priority'] ) && in_array( $rawGroup['priority'], [ 'high', 'core', 'default', 'low' ] ) ? $rawGroup['priority'] : 'default',
                    // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_key
                    'meta_key'  => !empty( $rawGroup['meta_key'] ) ? sanitize_key( wp_unslash( $rawGroup['meta_key'] ) ) : null,
                    'post_type' => $post_type,
                    'field'     => [],
                ];
                if ( empty( $_group['title'] ) || empty( $_group['meta_key'] ) ) {
                    continue;
                }
                if ( !empty( $rawGroup['field'] ) && is_array( $rawGroup['field'] ) ) {
                    foreach ( $rawGroup['field'] as $key => $value ) {
                        if ( is_array( $value ) && !empty( $value ) ) {
                            foreach ( $value as $k => $v ) {
                                if ( in_array( $k, $fieldOptions ) ) {
                                    if ( 'type' === $k ) {
                                        $_group['field'][$key][$k] = !empty( $v ) && in_array( $v, array_keys( $postMeta->getFields( 'key' ) ) ) ? $v : 'text';
                                    } else {
                                        $_group['field'][$key][$k] = !empty( $v ) ? sanitize_text_field( wp_unslash( $v ) ) : '';
                                    }
                                }
                            }
                        }
                    }
                }
                $groups[] = $_group;
            }
            $pm_options = get_option( $postMeta->options['post_meta'], [] );
            $pm_options = is_array( $pm_options ) ? $pm_options : [];
            if ( empty( $groups ) ) {
                if ( isset( $pm_options[$post_type] ) ) {
                    unset( $pm_options[$post_type] );
                }
            } else {
                $pm_options[$post_type]['group'] = $groups;
            }
            update_option( $postMeta->options['post_meta'], $pm_options );
            wp_send_json_success( [ 'msg' => $postMeta->showMessage( "Settings Successfully Saved", 'success' ) ] );
        }

    }
endif;