<?php

if ( !class_exists( 'pmPostTypeController' ) ) :
    class pmPostTypeController {

        function __construct() {
            add_action( 'admin_menu', [ $this, 'post_type_menu' ] );
            add_action( 'wp_ajax_pm_post_type_add', [ $this, 'pm_post_type_add' ] );
            add_action( 'wp_ajax_pm_post_type_edit', [ $this, 'pm_post_type_edit' ] );
            add_action( 'wp_ajax_pm_post_type_delete', [ $this, 'pm_post_type_delete' ] );
            add_action( 'wp_ajax_pm_post_type_update', [ $this, 'pm_post_type_update' ] );


            add_action( 'init', [ &$this, 'pm_register_post_types' ] );
        }

        function post_type_menu() {
            $page = add_submenu_page( 'post-meta', 'Manage Custom Post Types', 'Post Types', 'administrator', 'pm-manage-post-types', [ $this, 'post_types_menu_page' ] );
            add_action( 'admin_print_styles-' . $page, [ $this, 'post_types_sc' ] );
        }


        function post_types_menu_page() {
            global $postMeta;
            $postMeta->getRender( 'managePostTypes', '', 'posttype' );
        }

        function post_types_sc() {
            global $postMeta;
            wp_enqueue_script( 'jquery' );
            wp_enqueue_script( 'jquery-ui-core' );
            wp_enqueue_script( 'jquery-ui-widget' );
            wp_enqueue_script( 'jquery-ui-mouse' );
            wp_enqueue_script( 'jquery-ui-tabs' );
            wp_enqueue_style( 'jquery-ui-css', PM_ASSECTS_URL . 'css/jquery/jquery-ui.min.css', '', $postMeta->version );
            wp_enqueue_style( 'pluginCore-style', PM_ASSECTS_URL . 'css/pluginCore.css', '', $postMeta->version );
            wp_enqueue_style( 'post-meta-admin-style', PM_ASSECTS_URL . 'css/post_meta_admin.css', '', $postMeta->version );
            wp_enqueue_style( 'post-meta-validationEngine-css', PM_ASSECTS_URL . 'css/jquery/validationEngine.css', '', $postMeta->version );

            wp_enqueue_script( 'post-meta-validationEngine', PM_ASSECTS_URL . 'js/jquery/validationEngine.js', [ 'jquery' ], $postMeta->version, true );
            wp_enqueue_script( 'post-meta-validationEngine-en', PM_ASSECTS_URL . 'js/jquery/validationEngine-en.js', [ 'post-meta-validationEngine' ], $postMeta->version, true );
            wp_enqueue_script( 'post-meta-admin-script', PM_ASSECTS_URL . 'js/post_meta_admin.js', [ 'post-meta-validationEngine' ], $postMeta->version, true );
            wp_enqueue_script( 'post-meta-third-party-slug-string', PM_ASSECTS_URL . 'js/jquery/jquery.stringToSlug.js', [ 'jquery' ], $postMeta->version, true );
        }


        function pm_post_type_add() {
            global $postMeta;
            $postMeta->verifyNonce();
            $postMeta->render( 'postType', '', 'posttype' );
            die();
        }

        function pm_post_type_update() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            if ( wp_verify_nonce( $nonce, 'nonce' ) ) {
                unset( $_REQUEST['_wp_http_referer'] );
                unset( $_REQUEST['pm_nonce'] );
                unset( $_REQUEST['action'] );
                unset( $_REQUEST['post_type'] );

                // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.MissingUnslash, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
                $rawPostType = !empty( $_REQUEST['pm_posttype'] ) ? $_REQUEST['pm_posttype'] : [];
                $postType = [
                    'type'        => !empty( $rawPostType['type'] ) ? sanitize_key( sanitize_text_field( wp_unslash( $rawPostType['type'] ) ) ) : '',
                    'name'        => !empty( $rawPostType['name'] ) ? sanitize_text_field( wp_unslash( $rawPostType['name'] ) ) : '',
                    'label'       => !empty( $rawPostType['label'] ) ? sanitize_text_field( wp_unslash( $rawPostType['label'] ) ) : '',
                    'description' => !empty( $rawPostType['description'] ) ? sanitize_text_field( wp_unslash( $rawPostType['description'] ) ) : '',
                    'support'     => [],
                    'labels'      => [
                        'name'               => !empty( $rawPostType['labels']['name'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['name'] ) ) : '',
                        'singular_name'      => !empty( $rawPostType['labels']['singular_name'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['singular_name'] ) ) : '',
                        'add_new'            => !empty( $rawPostType['labels']['add_new'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['add_new'] ) ) : '',
                        'all_items'          => !empty( $rawPostType['labels']['all_items'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['all_items'] ) ) : '',
                        'add_new_item'       => !empty( $rawPostType['labels']['add_new_item'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['add_new_item'] ) ) : '',
                        'edit_item'          => !empty( $rawPostType['labels']['edit_item'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['edit_item'] ) ) : '',
                        'new_item'           => !empty( $rawPostType['labels']['new_item'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['new_item'] ) ) : '',
                        'view_item'          => !empty( $rawPostType['labels']['view_item'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['view_item'] ) ) : '',
                        'search_items'       => !empty( $rawPostType['labels']['search_items'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['search_items'] ) ) : '',
                        'not_found'          => !empty( $rawPostType['labels']['not_found'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['not_found'] ) ) : '',
                        'not_found_in_trash' => !empty( $rawPostType['labels']['not_found_in_trash'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['not_found_in_trash'] ) ) : '',
                        'parent_item_colon'  => !empty( $rawPostType['labels']['parent_item_colon'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['parent_item_colon'] ) ) : '',
                        'menu_name'          => !empty( $rawPostType['labels']['menu_name'] ) ? sanitize_text_field( wp_unslash( $rawPostType['labels']['menu_name'] ) ) : '',
                    ],
                    'option'      => [
                        'public'              => !empty( $rawPostType['option']['public'] ),
                        'publicly_queryable'  => !empty( $rawPostType['option']['public'] ),
                        'exclude_from_search' => !empty( $rawPostType['option']['public'] ),
                        'show_ui'             => !empty( $rawPostType['option']['public'] ),
                        'show_in_menu'        => !empty( $rawPostType['option']['public'] ),
                        'menu_position'       => !empty( $rawPostType['option']['menu_position'] ) ? sanitize_text_field( wp_unslash( $rawPostType['option']['menu_position'] ) ) : null,
                        'capability_type'     => !empty( $rawPostType['option']['capability_type'] ) ? sanitize_text_field( wp_unslash( $rawPostType['option']['capability_type'] ) ) : 'post',
                        'hierarchical'        => !empty( $rawPostType['option']['hierarchical'] ),
                        'has_archive'         => !empty( $rawPostType['option']['has_archive'] ),
                        'has_archive_slug'    => !empty( $rawPostType['option']['has_archive_slug'] ) ? sanitize_text_field( wp_unslash( $rawPostType['option']['has_archive_slug'] ) ) : null,
                        'rewrite'             => !empty( $rawPostType['option']['rewrite'] ),
                        'rewrite_slug'        => !empty( $rawPostType['option']['rewrite_slug'] ) ? sanitize_text_field( wp_unslash( $rawPostType['option']['rewrite_slug'] ) ) : null,
                        'query_var'           => !empty( $rawPostType['option']['query_var'] ),
                        'can_export'          => !empty( $rawPostType['option']['can_export'] ),
                        'show_in_nav_menus'   => !empty( $rawPostType['option']['show_in_nav_menus'] ),
                    ]
                ];

                if ( empty( $postType['type'] ) || empty( $postType['name'] ) ) {
                    wp_send_json_error( [
                        'msg' => $postMeta->showMessage( 'Missing required fields [post Type or, label]', 'error' )
                    ] );
                    return;
                }


                $pm_options = get_option( $postMeta->options['post_types'], [] );
                $pm_options = is_array( $pm_options ) ? $pm_options : [];
                $editKey = !empty( $_REQUEST['edit_key'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['edit_key'] ) ) : '';

                foreach ( $pm_options as $key => $pm_option ) {
                    if ( $pm_option['type'] === $postType['type'] ) {
                        if ( !empty( $_REQUEST['edit'] ) && isset( $editKey ) ) {
                            $pm_options[$key] = $postType;
                            update_option( $postMeta->options['post_types'], $pm_options );
                            $msg = $postMeta->showMessage( "Successfully Update", 'success' );
                            wp_send_json_success( [ 'msg' => $msg ] );
                            return;
                        }

                        $msg = $postMeta->showMessage( "This post type already exist", 'error' );
                        wp_send_json_error( [ 'msg' => $msg ] );
                        return;
                    }
                }

                if ( !empty( $_REQUEST['edit'] ) && isset( $editKey ) ) {
                    $pm_options[$postType['type']] = $postType;
                    update_option( $postMeta->options['post_types'], $pm_options );
                    $msg = $postMeta->showMessage( "Successfully Update", 'success' );
                    wp_send_json_success( [ 'msg' => $msg ] );
                    return;
                }
                $pm_options[$postType['type']] = $postType;
                update_option( $postMeta->options['post_types'], $pm_options );
                $msg = $postMeta->showMessage( "New Post Type Successfully Saved", 'success' );

                $menu = "<div class='pm_menu_option'>
                                <span><b>" . esc_html( $postType['name'] ) . "</b></span>
                                <span class='pm_manage_menu_option'>
                                    <a href='#Edit' rel='" . esc_attr( $postType['type'] ) . "' class='button' onclick='editPostType(this); return false;'>Edit</a> 
                                    <a href='#Delete' rel='" . esc_attr( $postType['type'] ) . "' class='button delete' onclick='deletePostType(this); return false;'>Delete</a>
                                </span>
                            </div>";

                wp_send_json_success( [ 'msg' => $msg, 'menu' => $menu ] );
            } else {
                $msg = $postMeta->showMessage( "Security Error", 'error' );
                wp_send_json_error( [ 'msg' => $msg ] );
            }

        }

        function pm_post_type_edit() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) ) die( 'Security check' );

            if ( !empty( $_REQUEST['post_type_key'] ) ) {
                $pm_options = get_option( $postMeta->options['post_types'] );
                $editKey = sanitize_text_field( wp_unslash( $_REQUEST['post_type_key'] ) );
                $postMeta->render( 'postType', [ 'data' => $pm_options[$editKey], 'editKey' => $editKey ], 'posttype' );
                die();
            }
        }

        function pm_post_type_delete() {
            global $postMeta;
            $nonce = isset( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) ) die( 'Security check' );
            if ( !empty( $_REQUEST['post_type_key'] ) ) {
                $pm_options = get_option( $postMeta->options['post_types'] );
                $deleteKey = sanitize_text_field( wp_unslash( $_REQUEST['post_type_key'] ) );
                unset( $pm_options[$deleteKey] );

                update_option( $postMeta->options['post_types'], $pm_options );
                PmFns::printMessage( $postMeta->showMessage( "Post Type Successfully Deleted", 'success' ) );
                die();
            } else {
                echo 'Error';
            }
        }


        function pm_register_post_types() {
            global $postMeta;

            $pm_postTypes = get_option( $postMeta->options['post_types'] );

            if ( $pm_postTypes ) {
                foreach ( $pm_postTypes as $postType ) {

                    $name = $postType['type'];
                    $postType['labels'] = !empty( $postType['labels'] ) && is_array( $postType['labels'] ) ? $postType['labels'] : [];
                    $option = [];
                    $postType['taxonomy'] = !empty( $postType['taxonomy'] ) && is_array( $postType['taxonomy'] ) ? $postType['taxonomy'] : [];
                    $postType['support'] = !empty( $postType['support'] ) && is_array( $postType['support'] ) ? $postType['support'] : [];
                    $option['public'] = isset( $postType['options']['public'] ) && $postType['options']['public'] == true;
                    $option['publicly_queryable'] = isset( $postType['options']['public'] ) && $postType['options']['publicly_queryable'] == true;
                    $option['exclude_from_search'] = isset( $postType['options']['public'] ) && $postType['options']['exclude_from_search'] == true;
                    $option['show_ui'] = isset( $postType['options']['public'] ) && $postType['options']['show_ui'] == true;
                    $option['show_in_menu'] = isset( $postType['options']['public'] ) && $postType['options']['show_in_menu'] == true;
                    $option['hierarchical'] = isset( $postType['options']['public'] ) && $postType['options']['hierarchical'] == true;
                    $option['has_archive'] = isset( $postType['options']['public'] ) && $postType['options']['has_archive'] == true;
                    $option['rewrite'] = isset( $postType['options']['public'] ) && $postType['options']['rewrite'] == true;
                    $option['query_var'] = isset( $postType['options']['public'] ) && $postType['options']['query_var'] == true;
                    $option['can_export'] = isset( $postType['options']['public'] ) && $postType['options']['can_export'] == true;
                    $option['show_in_nav_menus'] = isset( $postType['options']['public'] ) && $postType['options']['show_in_nav_menus'] == true;
                    $option['description'] = !empty( $postType['description'] ) ? $postType['description'] : '';


                    if ( isset( $option['has_archive'] ) && $option['has_archive'] && isset( $option['has_archive_slug'] ) && $option['has_archive_slug'] )
                        $option['has_archive'] = $option['has_archive_slug'];

                    if ( $option['rewrite'] && $option['rewrite_slug'] )
                        $option['rewrite'] = [ 'slug' => $option['rewrite_slug'], 'with_front' => true ];

                    //check Capability type
                    if ( empty( $postType['options']['capability_type'] ) ) {
                        $option['capability_type'] = 'post';
                    } elseif ( !in_array( $postType['options']['capability_type'], [ 'post', 'page' ] ) ) {
                        $option['capabilities'] = $this->_get_cap( $postType['options']['capability_type'] );
                    }
                    $option['labels'] = $postType['labels'];
                    $option['taxonomies'] = $postType['taxonomy'];
                    $option['supports'] = $postType['support'];


                    $args = [
                        'labels'             => $postType['labels'],
                        'public'             => $option['public'],
                        'publicly_queryable' => $option['publicly_queryable'],
                        'show_ui'            => $option['show_ui'],
                        'show_in_menu'       => $option['show_in_menu'],
                        'query_var'          => $option['query_var'],
                        'rewrite'            => $option['rewrite'],
                        'capability_type'    => 'post',
                        'has_archive'        => $option['has_archive'],
                        'hierarchical'       => $option['hierarchical'],
                        'menu_position'      => ( isset( $option['menu_position'] ) ) ? $option['menu_position'] : null,
                        'taxonomies'         => $postType['taxonomy'],
                        'supports'           => $postType['support'],
                        'can_export'         => $option['can_export'],
                        'description'        => $postType['description'],
                        'show_in_nav_menus'  => $option['show_in_nav_menus']
                    ];
                    register_post_type( $name, $args );
                }
            }

        }

        public function _get_cap( $name ) {
            return [
                'edit_post'          => sprintf( 'edit_%s', $name ),
                'read_post'          => sprintf( 'read_%s', $name ),
                'delete_post'        => sprintf( 'delete_%s', $name ),
                'edit_posts'         => sprintf( 'edit_%ss', $name ),
                'edit_others_posts'  => sprintf( 'edit_others_%ss', $name ),
                'publish_posts'      => sprintf( 'publish_%ss', $name ),
                'read_private_posts' => sprintf( 'read_private_%ss', $name )
            ];
        }
    }

endif;
