<?php

if ( !class_exists( 'pmPreloadsController' ) ) :
    class pmPreloadsController {

        function __construct() {
            global $postMeta;

            add_action( 'admin_print_scripts', [ $this, 'setJqueryVariable' ] );
            add_action( 'wp_print_scripts', [ $this, 'setJqueryVariable' ] );

            register_activation_hook( $postMeta->file, [ $this, 'postMetaActivation' ] );
            register_deactivation_hook( $postMeta->file, [ $this, 'postMetaDeActivation' ] );
        }

        function setJqueryVariable() {
            global $postMeta;
            $ajaxurl = admin_url( 'admin-ajax.php' );
            $nonceText = $postMeta->nonceText();
            $nonce = wp_create_nonce( $nonceText );

            if ( is_admin() )
                echo "<script type='text/javascript'>pm_nonce='" . esc_html( $nonce ) . "';</script>";
            else
                echo "<script type='text/javascript'>ajaxurl='" . esc_url( $ajaxurl ) . "';pm_nonce='" . esc_html( $nonce ) . "';</script>";
        }

        function postMetaActivation() {
            global $postMeta;
            $settings = get_option( $postMeta->options['settings'] );
            if ( !$settings ) {
                $data = [
                    'file' => [
                        'allext'        => [
                            'file'     => [ 'zip', 'rar', '7z', 'iso' ],
                            'document' => [ 'pdf', 'doc', 'ppt' ],
                            'image'    => [ 'jpg', 'jpeg', 'png', 'gif' ],
                            'audio'    => [ 'mp3', 'wav', 'wma' ],
                            'video'    => [ 'mp4', 'flv', 'avi', 'wmv' ]
                        ],
                        'max_file_size' => '200'
                    ]
                ];

                update_option( $postMeta->options['settings'], $data );
            }
            wp_schedule_event( current_time( 'timestamp' ), 'daily', 'post_meta_schedule_event' );
        }

        function postMetaDeActivation() {
            global $postMeta;
            delete_option( $postMeta->options['settings'] );
        }

    }
endif;
