<?php

if ( !class_exists( 'pmSettingsController' ) ) :
    class pmSettingsController {

        function __construct() {
            add_action( 'admin_menu', [ $this, 'settings_menu' ] );
            add_action( 'wp_ajax_pm_update_settings', [ $this, 'pm_update_settings' ] );
            add_action( 'wp_ajax_pm_reset_settings', [ $this, 'pm_reset_settings' ] );
        }

        function settings_menu() {
            $page = add_submenu_page( 'post-meta', 'Post Meta Settings', 'Settings', 'administrator', 'post-meta-settings', [ $this, 'settings_menu_page' ] );

            add_action( 'admin_print_styles-' . $page, [ $this, 'settings_sc' ] );
        }

        function settings_sc() {
            global $postMeta;
            wp_enqueue_script( 'jquery-ui-tabs' );
            wp_enqueue_style( 'pluginCore-style', PM_ASSECTS_URL . 'css/pluginCore.css', '', $postMeta->version );
            wp_enqueue_style( 'post-meta-admin-style', PM_ASSECTS_URL . 'css/post_meta_admin.css', '', $postMeta->version );
            wp_enqueue_style( 'post-meta-validationEngine-css', PM_ASSECTS_URL . 'css/jquery/validationEngine.css', '', $postMeta->version );
            wp_enqueue_style( 'jquery-ui-css', PM_ASSECTS_URL . 'css/jquery/jquery-ui.min.css', '', $postMeta->version );

            wp_enqueue_script( 'post-meta-validationEngine', PM_ASSECTS_URL . 'js/jquery/validationEngine.js', [ 'jquery' ], $postMeta->version, true );
            wp_enqueue_script( 'post-meta-validationEngine-en', PM_ASSECTS_URL . 'js/jquery/validationEngine-en.js', [ 'post-meta-validationEngine' ], $postMeta->version, true );
            wp_enqueue_script( 'post-meta-admin-script', PM_ASSECTS_URL . '/js/post_meta_admin.js', [ 'post-meta-validationEngine' ], $postMeta->version, true );

        }

        function settings_menu_page() {
            global $postMeta;

            $postMeta->getRender( "manageSettings", '', 'setting' );
        }

        function pm_update_settings() {
            global $postMeta;
            $nonce = !empty( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) || !current_user_can( 'manage_options' ) ) {
                wp_send_json_error( [ 'msg' => $postMeta->showMessage( 'Unauthorized action.', 'error' ) ] );
            }
            $settings = get_option( $postMeta->options['settings'], [] );
            $pmAllExts = $postMeta->pmAllExts();
            $settings['file']['allext_custom'] = !empty( $_REQUEST['settings']['file']['allext_custom'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['settings']['file']['allext_custom'] ) ) : '';
            if ( !empty( $_REQUEST['settings']['file']['allext'] ) && is_array( $_REQUEST['settings']['file']['allext'] ) ) {
                // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.MissingUnslash, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
                foreach ( $_REQUEST['settings']['file']['allext'] as $key => $value ) {
                    if ( is_array( $value ) ) {
                        $extensions = array_keys( $pmAllExts[$key] );
                        $settings['file']['allext'][$key] = array_filter( $value, function ( $_item ) use ( $extensions ) {
                            return in_array( $_item, $extensions );
                        } );
                    }
                }
            }
            if ( !empty( $_REQUEST['settings']['file']['max_file_size'] ) && in_array( $_REQUEST['settings']['file']['max_file_size'], [ 50, 100, 200 ] ) ) {
                $settings['file']['max_file_size'] = absint( $_REQUEST['settings']['file']['max_file_size'] );
            } else if ( !empty( $_REQUEST['settings']['file']['max_file_size_custom'] ) ) {
                $settings['file']['max_file_size_custom'] = absint( $_REQUEST['settings']['file']['max_file_size_custom'] );
            }
            update_option( $postMeta->options['settings'], $settings ); //Update existing custom Taxonomy array in the CPT option
            wp_send_json_success( [ 'msg' => $postMeta->showMessage( "Settings successfully update", 'success' ) ] );
        }

        function pm_reset_settings() {
            global $postMeta;
            $nonce = !empty( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) || !current_user_can( 'manage_options' ) ) {
                wp_send_json_error( [ 'msg' => $postMeta->showMessage( 'Unauthorized action.', 'error' ) ] );
            }
            $data = [
                'file' => [
                    'allext'        => [
                        'file'     => [ 'zip', 'rar', '7z', 'iso' ],
                        'document' => [ 'pdf', 'doc', 'ppt' ],
                        'image'    => [ 'jpg', 'jpeg', 'png', 'gif' ],
                        'audio'    => [ 'mp3', 'wav', 'wma' ],
                        'video'    => [ 'mp4', 'flv', 'avi', 'wmv' ]
                    ],
                    'max_file_size' => '200'
                ]
            ];

            update_option( $postMeta->options['settings'], $data );
            wp_send_json_success( [ 'msg' => $postMeta->showMessage( "Settings successfully Restored to default", 'success' ) ] );
        }
    }

endif;
