<?php

if ( !class_exists( 'pmTaxonomyController' ) ) :
    class pmTaxonomyController {

        function __construct() {
            add_action( 'admin_menu', [ $this, 'pm_menu' ] );
            add_action( 'wp_ajax_pm_taxonomy_add', [ $this, 'pm_taxonomy_add' ] );
            add_action( 'wp_ajax_pm_taxonomy_update', [ $this, 'pm_taxonomy_update' ] );
            add_action( 'wp_ajax_pm_taxonomy_edit', [ $this, 'pm_taxonomy_edit' ] );
            add_action( 'wp_ajax_pm_taxonomy_delete', [ $this, 'pm_taxonomy_delete' ] );
            add_action( 'init', [ &$this, 'pm_register_taxonomies' ] );
        }

        function pm_menu() {
            $page = add_submenu_page( 'post-meta', 'Manage Custom Taxonomies', 'Taxonomies', 'administrator', 'manage-taxonomies', [ $this, 'taxonomies_menu_page' ] );
            add_action( 'admin_print_styles-' . $page, [ $this, 'taxonomies_sc' ] );
        }

        function taxonomies_menu_page() {
            global $postMeta;

            $postMeta->getRender( 'mamageTaxonomies', '', 'taxonomy' );
        }

        function taxonomies_sc() {
            global $postMeta;
            wp_enqueue_script( 'jquery' );
            wp_enqueue_script( 'jquery-ui-core' );
            wp_enqueue_script( 'jquery-ui-widget' );
            wp_enqueue_script( 'jquery-ui-mouse' );
            wp_enqueue_script( 'jquery-ui-tabs' );
            wp_enqueue_style( 'jquery-ui-css', PM_ASSECTS_URL . 'css/jquery/jquery-ui.min.css', '', $postMeta->version );
            wp_enqueue_style( 'pluginCore-style', PM_ASSECTS_URL . 'css/pluginCore.css', '', $postMeta->version );
            wp_enqueue_style( 'post-meta-admin-style', PM_ASSECTS_URL . 'css/post_meta_admin.css', '', $postMeta->version );
            wp_enqueue_style( 'post-meta-validationEngine-css', PM_ASSECTS_URL . 'css/jquery/validationEngine.css', '', $postMeta->version );

            wp_enqueue_script( 'post-meta-validationEngine', PM_ASSECTS_URL . 'js/jquery/validationEngine.js', [ 'jquery' ], $postMeta->version, true );
            wp_enqueue_script( 'post-meta-validationEngine-en', PM_ASSECTS_URL . 'js/jquery/validationEngine-en.js', [ 'post-meta-validationEngine' ], $postMeta->version, true );
            wp_enqueue_script( 'post-meta-third-party-slug-string', PM_ASSECTS_URL . 'js/jquery/jquery.stringToSlug.js', '', $postMeta->version, true );
            wp_enqueue_script( 'post-meta-admin-script', PM_ASSECTS_URL . 'js/post_meta_admin.js', [ 'post-meta-validationEngine' ], $postMeta->version, true );
        }

        function pm_taxonomy_add() {
            global $postMeta;
            $nonce = !empty( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) || !current_user_can( 'manage_options' ) ) {
                PmFns::printMessage( $postMeta->showMessage( 'Unauthorized action.', 'error' ) );
                die();
            }
            $postMeta->render( 'taxonomy', '', 'taxonomy' );
            die();
        }


        function pm_taxonomy_update() {
            global $postMeta;

            $nonce = !empty( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) || !current_user_can( 'manage_options' ) ) {
                wp_send_json_error( [ 'msg' => $postMeta->showMessage( 'Unauthorized action.', 'error' ) ] );
            }

            unset( $_REQUEST['_wp_http_referer'] );
            unset( $_REQUEST['pm_nonce'] );
            unset( $_REQUEST['action'] );

            // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.MissingUnslash, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
            $rawTaxonomy = !empty( $_REQUEST['pm_taxonomy'] ) && is_array( $_REQUEST['pm_taxonomy'] ) ? $_REQUEST['pm_taxonomy'] : [];
            $taxonomy = [];
            $fieldOptions = [
                'type'        => '',
                'name'        => '',
                'label'       => '',
                'description' => '',
                'post_types'  => '',
                'labels'      => [
                    'name',
                    'singular_name',
                    'search_items',
                    'edit_items',
                    'add_new_item',
                    'all_items',
                    'parent_item',
                    'parent_item_colon',
                    'edit_item',
                    'update_item',
                    'add_new_item',
                    'separate_items_with_commas',
                    'add_or_remove_items',
                    'choose_from_most_used',
                    'menu_name'
                ],
                'option'      => [
                    'boolean' => [
                        'public',
                        'show_in_nav_menus',
                        'show_ui',
                        'show_tagcloud',
                        'hierarchical',
                        'rewrite',
                        'query_var'
                    ],
                    'string'  => [
                        'update_count_callback',
                        'rewrite_slug',
                    ]
                ]
            ];
            foreach ( $rawTaxonomy as $key => $value ) {
                if ( in_array( $key, array_keys( $fieldOptions ) ) ) {
                    if ( 'type' == $key ) {
                        $taxonomy[$key] = !empty( $value ) ? sanitize_key( wp_unslash( $value ) ) : null;
                    } else if ( 'post_types' == $key ) {
                        if ( is_array( $value ) ) {
                            $types = [];
                            foreach ( $value as $post_type ) {
                                if ( post_type_exists( $post_type ) ) {
                                    $types[] = $post_type;
                                }
                            }
                            $taxonomy[$key] = $types;
                        }
                    } else if ( 'labels' == $key ) {
                        if ( is_array( $value ) ) {
                            $labels = [];
                            foreach ( $value as $k => $label ) {
                                if ( in_array( $k, $fieldOptions['labels'] ) ) {
                                    $labels[$k] = !empty( $label ) ? sanitize_text_field( wp_unslash( $label ) ) : null;
                                }
                            }
                            $taxonomy[$key] = $labels;
                        }
                    } else if ( 'option' == $key ) {
                        if ( is_array( $value ) ) {
                            $options = [];
                            foreach ( $value as $k => $option ) {
                                if ( in_array( $k, $fieldOptions['option']['boolean'] ) ) {
                                    $options[$k] = !empty( $option );
                                }
                                if ( in_array( $k, $fieldOptions['option']['string'] ) ) {
                                    $options[$k] = !empty( $option ) ? sanitize_text_field( wp_unslash( $option ) ) : null;
                                }
                            }
                            $taxonomy[$key] = $options;
                        }
                    } else {
                        $taxonomy[$key] = !empty( $value ) ? sanitize_text_field( wp_unslash( $value ) ) : null;
                    }
                }
            }

            if ( empty( $taxonomy['type'] ) || empty( $taxonomy['post_types'] ) || empty( $taxonomy['labels'] ) ) {
                wp_send_json_error( [ 'msg' => $postMeta->showMessage( "Some required field is missing", 'error' ) ] );
            }
            $editKey = !empty( $_REQUEST['edit_key'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['edit_key'] ) ) : '';
            
            $pm_options = get_option( $postMeta->options['taxonomies'] );
            $pm_options = !is_array( $pm_options ) ? [] : $pm_options;
            foreach ( $pm_options as $key => $pm_option ) {
                if ( $pm_option['type'] == $taxonomy['type'] ) {
                    if ( !empty( $_REQUEST['edit'] ) && isset( $editKey ) ) {
                        $pm_options[$key] = $taxonomy;
                        update_option( $postMeta->options['taxonomies'], $pm_options ); //Update existing custom Taxonomy array in the CPT option
                        $msg = $postMeta->showMessage( "Taxonomy successfully update", 'success' );
                        wp_send_json_success( [ 'msg' => $msg ] );
                    }

                    $msg = $postMeta->showMessage( "This custom taxonomy already exist", 'error' );
                    wp_send_json_error( [ 'msg' => $msg ] );
                }

            }
            if ( !empty( $_REQUEST['edit'] ) && isset( $editKey ) ) {
                $pm_options[$editKey] = $taxonomy;
                update_option( $postMeta->options['taxonomies'], $pm_options ); //Update existing custom Taxonomy if type name can ne change
                $msg = $postMeta->showMessage( "Taxonomy successfully update", 'success' );
                wp_send_json_success( [ 'msg' => $msg ] );
            }


            $pm_options[$taxonomy['type']] = $taxonomy; //insert new custom taxonomy into the array
            update_option( $postMeta->options['taxonomies'], $pm_options );  //save new custom Taxonomy array in the CPT option

            $msg = $postMeta->showMessage( "Taxonomy successfully saved", 'success' );
            $menu = "<div class='pm_menu_option'>
                                <span><b>{$taxonomy['name']}</b></span>
                                <span class='pm_manage_menu_option'>
                                    <a href='#Edit' rel='{$taxonomy['type']}' class='button' onclick='editTaxonomy(this); return false;'>Edit</a> 
                                    <a href='#Delete' rel='{$taxonomy['type']}' class='button delete' onclick='deleteTaxonomy(this); return false;'>Delete</a>
                                </span>
                            </div>";
            wp_send_json_success( [ 'msg' => $msg, 'menu' => $menu ] );
        }

        function pm_taxonomy_edit() {
            global $postMeta;
            $nonce = !empty( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) || !current_user_can( 'manage_options' ) ) die( 'Security check' );
            if ( !empty( $_REQUEST['taxonomy_key'] ) ) {
                $pm_options = get_option( $postMeta->options['taxonomies'] );
                $editKey = sanitize_text_field( wp_unslash( $_REQUEST['taxonomy_key'] ) );
                $postMeta->render( 'taxonomy', [ 'data' => $pm_options[$editKey], 'editKey' => $editKey ], 'taxonomy' );
                die();
            }
        }

        function pm_taxonomy_delete() {
            global $postMeta;
            $nonce = !empty( $_REQUEST['pm_nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['pm_nonce'] ) ) : '';
            $nonceText = $postMeta->nonceText();
            if ( !wp_verify_nonce( $nonce, $nonceText ) || !current_user_can( 'manage_options' ) ) die( 'Security check' );
            if ( !empty( $_REQUEST['taxonomy_key'] ) ) {
                $pm_options = get_option( $postMeta->options['taxonomies'] );
                $deleteKey = sanitize_text_field( wp_unslash( $_REQUEST['taxonomy_key'] ) );
                unset( $pm_options[$deleteKey] );

                update_option( $postMeta->options['taxonomies'], $pm_options );
                PmFns::printMessage( $postMeta->showMessage( "Taxonomy Successfully Deleted", 'success' ) );
                die();
            } else {
                echo 'Error';
            }
        }

        function pm_register_taxonomies() {
            global $postMeta;
            $taxonomies = get_option( $postMeta->options['taxonomies'] );

            if ( $taxonomies ) {
                foreach ( $taxonomies as $taxonomy ) {
                    $name = $taxonomy['type'];
                    $in = $taxonomy['post_types'];
                    $option = $taxonomy['option'];
                    $option['labels'] = $taxonomy['labels'];
                    $option['public'] = $option['public'] == true;
                    $option['show_in_nav_menus'] = $option['show_in_nav_menus'] == true;
                    $option['show_ui'] = $option['show_ui'] == true;
                    $option['show_tagcloud'] = !empty( $option['show_tagcloud'] );
                    $option['hierarchical'] = $option['hierarchical'] == true;
                    $option['query_var'] = $option['query_var'] == true;


                    if ( !$option['update_count_callback'] ) {
                        unset( $option['update_count_callback'] );
                    }
                    if ( !empty($option['rewrite']) && !empty($option['rewrite_slug']) ) {
                        $option['rewrite'] = [ 'slug' => $option['rewrite_slug'] ];
                    }
                    unset( $option['rewrite_slug'] );
                    register_taxonomy( $name, $in, $option ); // Register taxonomy
                }

            }


        }

    }

endif;
