<?php
/**
 * Plugin Name: Post Meta
 * Plugin URI: http://itemcoder.com
 * Description: Post Meta is a wordpress custom post field, post type and taxonomy management Plugin. It has smart and modern (ajax and jquery based) interface to create post meta option or custom meta field as group or field.
 * Author: Mahbubur Rahman
 * Version: 1.1.0
 * Author URI: http://itemcoder.com
 * Text Domain: post-meta
 * Domain Path: /languages/
 * Requires at least: 6.6
 * Requires PHP: 7.4
 * License: GPLv3 or later
 * License URI: https://www.gnu.org/licenses/gpl-3.0.html
 *
 * @package post-meta
 */

defined( 'ABSPATH' ) || exit;

require_once( 'lib/init.php' );
require_once( 'lib/api.php' );

if ( !class_exists( 'postMeta' ) ) {
    class postMeta extends pluginCore {

        public $title = 'Post Meta';
        public $name = 'post-meta';
        public $version = '1.1.0';
        public $prefix = 'pm_';
        public $prefixLong = 'post_meta_';
        public $website = 'https://itemcoder.com';
        public string $file;
        public string $pluginSlug;
        public string $pluginPath;
        public string $modelsPath;
        public string $controllersPath;
        public string $viewsPath;
        public string $helperPath;
        public string $pluginUrl;
        public string $assetsUrl;
        public string $helperUrl;
        /**
         * @var array|string[]
         */
        public array $options;


        function __construct() {
            $this->file = __FILE__;
            $this->pluginSlug = plugin_basename( __FILE__ );
            $this->pluginPath = dirname( __FILE__ );
            $this->modelsPath = $this->pluginPath . '/lib/models/';
            $this->controllersPath = $this->pluginPath . '/lib/controllers/';
            $this->viewsPath = $this->pluginPath . '/lib/views/';
            $this->helperPath = $this->pluginPath . '/lib/helpers/';

            $this->pluginUrl = plugins_url( '', __FILE__ );
            $this->assetsUrl = $this->pluginUrl . '/assets/';
            $this->helperUrl = $this->pluginUrl . '/lib/helpers/';
            define( 'PM_PATH', dirname( __FILE__ ) );
            define( 'PM_PLUGIN_URL', plugins_url( '', __FILE__ ) ) . '/';
            define( 'PM_ASSECTS_URL', PM_PLUGIN_URL . '/assets/' );
            define( 'PM_HELPER_URL', PM_PLUGIN_URL . '/lib/helpers/' );


            $this->loadModels( $this->modelsPath );
            $this->loadModels( $this->modelsPath . 'enc/', true );

            $this->options = [
                'post_meta'  => 'pm_post_meta',
                'post_types' => 'pm_post_types',
                'taxonomies' => 'pm_taxonomies',
                'settings'   => 'pm_settings',
                'cache'      => 'pm_cache'
            ];


        }

        function loadControllers() {
            global $postMeta;
            $controllersOrder = $postMeta->controllersOrder();

            $classes = [];
            foreach ( scandir( $postMeta->controllersPath ) as $file ) {
                if ( preg_match( "/.php$/i", $file ) )
                    $classes[str_replace( ".php", "", $file )] = $postMeta->controllersPath . $file;
            }

            foreach ( $classes as $className => $classPath ) {
                require_once( $classPath );
                if ( !in_array( $className, $controllersOrder ) )
                    $controllersOrder[] = $className;
            }

            foreach ( $controllersOrder as $className ) {
                if ( class_exists( $className ) )
                    $instance[] = new $className;
            }

            return $instance;
        }


        function render( $viewName, $parameter = [], $subdir = null ) {
            // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
            echo self::getRender( $viewName, $parameter, $subdir );
        }

        function getRender( $viewName, $parameter = [], $subdir = null ) {
            global $postMeta;

            $subdir = $subdir ? "$subdir/" : null;
            if ( !empty( $postMeta->isPro ) && self::isPro() and file_exists( $postMeta->viewsPath . "pro/$subdir" . $viewName . '.php' ) )
                $viewPath = $postMeta->viewsPath . "pro/$subdir" . $viewName . '.php';
            else {
                $viewPath = $postMeta->viewsPath . $subdir . $viewName . '.php';
                if ( !file_exists( $viewPath ) ) return;
            }

            if ( $parameter ) extract( $parameter );
            $pageReturn = include $viewPath;
            if ( $pageReturn and $pageReturn <> 1 )
                return $pageReturn;
            if ( !empty( $html ) ) {
                return $html;
            }
        }

        function isPro(): bool {
            return class_exists( 'postMetaPro' );
        }

        function init() {
            $this->loadControllers();
            $this->loadDirectory( $this->helperPath );
        }
    }

    global $postMeta;
    $postMeta = new postMeta;
    $postMeta->init();
}
